<?php

/* --------------------------------------------------------------
   ThemeDirectories.php 2019-04-11
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the MIT License
   [https://opensource.org/licenses/MIT]
   --------------------------------------------------------------
*/

/**
 * Class ThemeDirectories
 */
class ThemeDirectories implements ThemeDirectoriesInterface
{
    /**
     * Directory root
     *
     * @var ThemeDirectoryRootInterface
     */
    protected $root;

    /**
     * Configuration directory.
     *
     * @var \ThemeDirectory
     */
    protected $config;

    /**
     * HTML directory
     *
     * @var ThemeDirectory|null
     */
    protected $html;

    /**
     * JavaScript directory
     *
     * @var ThemeDirectory|null
     */
    protected $js;

    /**
     * Styles directory
     *
     * @var ThemeDirectory|null
     */
    protected $styles;

    /**
     * Style Edit directory.
     *
     * @var ThemeDirectory|null
     */
    protected $styleEdit;

    /**
     * Images directory
     *
     * @var ThemeDirectory|null
     */
    protected $images;
    
    /**
     * Fonts directory
     *
     * @var ThemeDirectory|null
     */
    protected $fonts;

    /**
     * Custom HTML directory
     *
     * @var ThemeDirectory|null
     */
    protected $customHtml;

    /**
     * Custom JS directory
     *
     * @var ThemeDirectory|null
     */
    protected $customJs;

    /**
     * Custom styles directory
     *
     * @var ThemeDirectory|null
     */
    protected $customStyles;
    
    /**
     * ThemeDirectories constructor.
     *
     * @param \ThemeDirectoryRootInterface $root Path to root directory of theme.
     */
    public function __construct(ThemeDirectoryRootInterface $root)
    {
        $this->root = $root;
        $this->config = ThemeDirectory::create($root->withPath('config'));

        $this->html = $this->tryBuildThemeDirectory($root, 'html' . DIRECTORY_SEPARATOR . 'system');
        $this->customHtml = $this->tryBuildThemeDirectory($root, 'html' . DIRECTORY_SEPARATOR . 'custom');

        $this->js = $this->tryBuildThemeDirectory($root, 'javascripts' . DIRECTORY_SEPARATOR . 'system');
        $this->customJs = $this->tryBuildThemeDirectory($root, 'javascripts' . DIRECTORY_SEPARATOR . 'custom');

        $this->styles = $this->tryBuildThemeDirectory($root, 'styles' . DIRECTORY_SEPARATOR . 'system');
        $this->styleEdit = $this->tryBuildThemeDirectory($root, 'styles' . DIRECTORY_SEPARATOR . 'styleedit');
        $this->customStyles = $this->tryBuildThemeDirectory($root, 'styles' . DIRECTORY_SEPARATOR . 'custom');

        $this->images = $this->tryBuildThemeDirectory($root, 'images');
        $this->fonts = $this->tryBuildThemeDirectory($root, 'fonts');
    }


    /**
     * Named constructor of ThemeDirectories.
     *
     * @param \ThemeDirectoryRootInterface $root Path to root directory of theme.
     *
     * @return \ThemeDirectories New instance.
     */
    public static function create(ThemeDirectoryRootInterface $root)
    {
        return MainFactory::create(static::class, $root);
    }


    /**
     * Tries to create an instance of ThemeDirectory by the given root- and sub path.
     * If no directory was found, null will be returned.
     *
     * @param \ThemeDirectoryRootInterface $root Path to root directory of theme.
     * @param string $path Sub directory path.
     *
     * @return null|\ThemeDirectory Theme directory instance if given path exists.
     */
    protected function tryBuildThemeDirectory(ThemeDirectoryRootInterface $root, $path)
    {
        $a = rtrim($root->getPath(), DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR . $path;

        return is_dir($a) ? ThemeDirectory::create($root->withPath($path)) : null;
    }


    /**
     * Return the directory root
     *
     * @return ThemeDirectoryRootInterface
     */
    public function getRoot()
    {
        return $this->root;
    }


    /**
     * Returns the config theme directory.
     *
     * @return \ThemeDirectoryInterface
     */
    public function getConfig()
    {
        return $this->config;
    }


    /**
     * Return the HTML directory
     *
     * @return ThemeDirectory|null
     */
    public function getHtml()
    {
        return $this->html;
    }


    /**
     * Return the scripts directory
     *
     * @return ThemeDirectory|null
     */
    public function getJs()
    {
        return $this->js;
    }


    /**
     * Return the styles directory
     *
     * @return ThemeDirectory|null
     */
    public function getStyles()
    {
        return $this->styles;
    }


    /**
     * Returns the style edit theme directory.
     *
     * @return \ThemeDirectory|null
     */
    public function getStyleEdit()
    {
        return $this->styleEdit;
    }


    /**
     * Returns the images directory.
     *
     * @return \ThemeDirectory|null
     */
    public function getImages()
    {
        return $this->images;
    }
    
    /**
     * Returns the fonts directory.
     *
     * @return \ThemeDirectory|null
     */
    public function getFonts()
    {
        return $this->fonts;
    }


    /**
     * Return the custom HTML directory
     *
     * @return ThemeDirectory|null
     */
    public function getCustomHtml()
    {
        return $this->customHtml;
    }


    /**
     * Return the custom scripts directory
     *
     * @return ThemeDirectory|null
     */
    public function getCustomJs()
    {
        return $this->customJs;
    }


    /**
     * Return the custom styles directory
     *
     * @return ThemeDirectory|null
     */
    public function getCustomStyles()
    {
        return $this->customStyles;
    }
}