<?php
/* --------------------------------------------------------------
   ThemeControl.inc.php 2018-11-19
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class ThemeControl
{
    /**
     * @var \CI_DB_query_builder
     */
    protected $db;

    /**
     * @var string
     */
    protected $currentTemplate;

    /**
     * @var bool
     */
    protected $themeSystemActive;

    /**
     * @var string
     */
    protected $currentTheme;

    /**
     * @var string
     */
    protected $themesPath;

    /**
     * @var string
     */
    protected $themePath;

    /**
     * @var string
     */
    protected $themeHtmlPath;

    /**
     * @var string
     */
    protected $themeSmartyPath;

    /**
     * @var string
     */
    protected $themeCssPath;

    /**
     * @var string
     */
    protected $themeJsPath;

    /**
     * @var string
     */
    protected $themeImagePath;

    /**
     * @var string
     */
    protected $themeFontsPath;

    /**
     * @var string
     */
    protected $themeConfigPath;

    /**
     * @var string
     */
    protected $categoryListingTemplatePath;

    /**
     * @var string
     */
    protected $filterSectionTemplatePath;

    /**
     * @var string
     */
    protected $productInfoTemplatePath;

    /**
     * @var string
     */
    protected $productListingTemplatePath;

    /**
     * @var string
     */
    protected $productOptionsTemplatePath;

    /**
     * @var string
     */
    protected $gmProductOptionsTemplatePath;

    /**
     * @var string
     */
    protected $propertiesTemplatePath;

    /**
     * @var string
     */
    protected $dynamicCssFilePath;

    /**
     * @var string
     */
    protected $templateSettingsFilePath;

    /**
     * @var array
     */
    protected $themeSettings;

    /**
     * @var float
     */
    protected $themeVersion;


    /**
     * ThemeControl constructor.
     *
     * @param \CI_DB_query_builder $db
     * @param string $currentTemplate
     */
    public function __construct(CI_DB_query_builder $db, $currentTemplate)
    {
        $this->db = $db;
        $this->currentTemplate = $currentTemplate;
    }


    /**
     * Returns the status of the theme system. True, if theme system is active, otherwise false.
     *
     * @return bool
     */
    public function isThemeSystemActive()
    {
        if ($this->themeSystemActive === null) {
            $this->_determineCurrentThemeStatus();
        }

        return $this->themeSystemActive;
    }


    /**
     * Returns the current theme name. If the theme system is not active, the current template name will be returned.
     *
     * @return string
     */
    public function getCurrentTheme()
    {
        if ($this->currentTheme === null) {
            $this->_determineCurrentThemeStatus();
        }

        return $this->currentTheme;
    }


    /**
     * Returns the current theme hierarchy. If the theme system is not active, the current template name will be
     * returned as array.
     *
     * @return array
     */
    public function getCurrentThemeHierarchy()
    {
        if (!$this->isThemeSystemActive()) {
            return (array)$this->currentTheme;
        }

        // TODO SPRINT #42: HIER SOLLTE SPÄTER EINMAL DER THEME SERVICE VERWENDET WERDEN

        $hierarchy = [];
        $parentTheme = $this->getCurrentTheme();

        while ($parentTheme !== null) {
            $hierarchy[] = $parentTheme;
            $themeJsonFile = DIR_FS_CATALOG . $this->getThemesPath() . $parentTheme . '/theme.json';
            $parentTheme = null;

            if (file_exists($themeJsonFile)) {
                $themeJson = json_decode(file_get_contents($themeJsonFile), true);
                if (isset($themeJson['extends'])) {
                    $parentTheme = $themeJson['extends'];
                }
            }
        }

        return $hierarchy;
    }


    /**
     * Returns the path to the themes, based on the shop root directory.
     *
     * @return string
     */
    public function getThemesPath()
    {
        if ($this->themesPath === null) {
            $this->themesPath = 'templates/';
            if ($this->isThemeSystemActive()) {
                $this->themesPath = 'themes/';
            }
        }

        return $this->themesPath;
    }


    /**
     * Returns the path to the theme, based on the shop root directory.
     *
     * @return string
     */
    public function getThemePath()
    {
        if ($this->themePath === null) {
            $this->themePath = 'templates/' . $this->currentTemplate . '/';
            if ($this->isThemeSystemActive()) {
                $this->themePath = 'public/theme/';
            }
        }

        return $this->themePath;
    }


    /**
     * Returns the path to the theme html directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeHtmlPath()
    {
        if ($this->themeHtmlPath === null) {
            $this->themeHtmlPath = 'templates/' . $this->currentTemplate . '/';
            if ($this->isThemeSystemActive()) {
                $this->themeHtmlPath = 'public/theme/html/system/';
            }
        }

        return $this->themeHtmlPath;
    }


    /**
     * Returns the path to the theme smarty directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeSmartyPath()
    {
        if ($this->themeSmartyPath === null) {
            $this->themeSmartyPath = 'templates/' . $this->currentTemplate . '/smarty/';
            if ($this->isThemeSystemActive()) {
                $this->themeSmartyPath = 'public/theme/html/smarty/';
            }
        }

        return $this->themeSmartyPath;
    }


    /**
     * Returns the path to the theme css directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeCssPath()
    {
        if ($this->themeCssPath === null) {
            $this->themeCssPath = 'templates/' . $this->currentTemplate . '/assets/styles/';
            if ($this->isThemeSystemActive()) {
                $this->themeCssPath = 'public/theme/styles/system/';
            }
        }

        return $this->themeCssPath;
    }


    /**
     * Returns the path to the theme javascript directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeJsPath()
    {
        if ($this->themeJsPath === null) {
            $this->themeJsPath = 'templates/' . $this->currentTemplate . '/assets/javascript/';
            if ($this->isThemeSystemActive()) {
                $this->themeJsPath = 'public/theme/javascripts/system/';
            }
        }

        return $this->themeJsPath;
    }


    /**
     * Returns the path to the theme image directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeImagePath()
    {
        if ($this->themeImagePath === null) {
            $this->themeImagePath = 'templates/' . $this->currentTemplate . '/assets/images/';
            if ($this->isThemeSystemActive()) {
                $this->themeImagePath = 'public/theme/images/';
            }
        }

        return $this->themeImagePath;
    }


    /**
     * Returns the path to the theme image directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeFontsPath()
    {
        if ($this->themeFontsPath === null) {
            $this->themeFontsPath = 'templates/' . $this->currentTemplate . '/assets/fonts/';
            if ($this->isThemeSystemActive()) {
                $this->themeFontsPath = 'public/theme/fonts/';
            }
        }

        return $this->themeFontsPath;
    }


    /**
     * Returns the path to the theme config directory, based on the shop root directory.
     *
     * @return string
     */
    public function getThemeConfigPath()
    {
        if ($this->themeConfigPath === null) {
            $this->themeConfigPath = 'templates/' . $this->currentTemplate . '/';
            if ($this->isThemeSystemActive()) {
                $this->themeConfigPath = 'public/theme/config/';
            }
        }

        return $this->themeConfigPath;
    }


    /**
     * Returns the path to the category listing templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getCategoryListingTemplatePath()
    {
        if ($this->categoryListingTemplatePath === null) {
            $this->categoryListingTemplatePath = 'templates/' . $this->currentTemplate . '/module/categorie_listing/';
            if ($this->isThemeSystemActive()) {
                $this->categoryListingTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->categoryListingTemplatePath;
    }


    /**
     * Returns the path to the category listing templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getFilterSelectionTemplatePath()
    {
        if ($this->filterSectionTemplatePath === null) {
            $this->filterSectionTemplatePath = 'templates/' . $this->currentTemplate . '/module/filter_selection/';
            if ($this->isThemeSystemActive()) {
                $this->filterSectionTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->filterSectionTemplatePath;
    }


    /**
     * Returns the path to the product info templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getProductInfoTemplatePath()
    {
        if ($this->productInfoTemplatePath === null) {
            $this->productInfoTemplatePath = 'templates/' . $this->currentTemplate . '/module/product_info/';
            if ($this->isThemeSystemActive()) {
                $this->productInfoTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->productInfoTemplatePath;
    }


    /**
     * Returns the path to the product listing templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getProductListingTemplatePath()
    {
        if ($this->productListingTemplatePath === null) {
            $this->productListingTemplatePath = 'templates/' . $this->currentTemplate . '/module/product_listing/';
            if ($this->isThemeSystemActive()) {
                $this->productListingTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->productListingTemplatePath;
    }


    /**
     * Returns the path to the product options templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getProductOptionsTemplatePath()
    {
        if ($this->productOptionsTemplatePath === null) {
            $this->productOptionsTemplatePath = 'templates/' . $this->currentTemplate . '/module/product_options/';
            if ($this->isThemeSystemActive()) {
                $this->productOptionsTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->productOptionsTemplatePath;
    }


    /**
     * Returns the path to the gm product option templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getGmProductOptionsTemplatePath()
    {
        if ($this->gmProductOptionsTemplatePath === null) {
            $this->gmProductOptionsTemplatePath = 'templates/' . $this->currentTemplate . '/module/gm_product_options/';
            if ($this->isThemeSystemActive()) {
                $this->gmProductOptionsTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->gmProductOptionsTemplatePath;
    }


    /**
     * Returns the path to the properties templates directory, based on the shop root directory.
     *
     * @return string
     */
    public function getPropertiesTemplatePath()
    {
        if ($this->propertiesTemplatePath === null) {
            $this->propertiesTemplatePath = 'templates/' . $this->currentTemplate . '/module/properties/';
            if ($this->isThemeSystemActive()) {
                $this->propertiesTemplatePath = 'public/theme/html/system/';
            }
        }

        return $this->propertiesTemplatePath;
    }


    /**
     * Returns the path to the dynamic css file, based on the shop root directory.
     *
     * @return string
     */
    public function getDynamicCssFilePath()
    {
        if ($this->dynamicCssFilePath === null) {
            $this->dynamicCssFilePath = 'templates/' . $this->currentTemplate . '/gm_dynamic.css.php';
            if ($this->isThemeSystemActive()) {
                $this->dynamicCssFilePath = 'templates/' . $this->currentTemplate . '/gm_dynamic.css.php';
            }
        }

        return $this->dynamicCssFilePath;
    }


    /**
     * Returns the path to the template settings file, based on the shop root directory.
     *
     * @return string
     */
    public function getTemplateSettingsFilePath()
    {
        if ($this->templateSettingsFilePath === null) {
            $this->templateSettingsFilePath = 'templates/' . $this->currentTemplate . '/template_settings.php';
            if ($this->isThemeSystemActive()) {
                $this->templateSettingsFilePath = 'public/theme/config/theme_settings.php';
            }
        }

        return $this->templateSettingsFilePath;
    }


    /**
     * Returns the path to the template settings file, based on the shop root directory.
     *
     * @return array
     */
    public function getThemeSettings()
    {
        if ($this->themeSettings === null) {
            $this->themeSettings = [];
            if ($this->isThemeSystemActive() && file_exists(DIR_FS_CATALOG . $this->getThemeConfigPath() . 'theme_settings.php')) {
                $themeSettingsArray = [];
                include(DIR_FS_CATALOG . $this->getThemeConfigPath() . 'theme_settings.php');

                $this->themeSettings = $themeSettingsArray;
            } elseif (file_exists(DIR_FS_CATALOG . $this->getThemeConfigPath() . 'template_settings.php')) {
                $t_template_settings_array = [];
                include(DIR_FS_CATALOG . $this->getThemeConfigPath() . 'template_settings.php');

                $this->themeSettings = $t_template_settings_array;
            }
        }

        return $this->themeSettings;
    }


    /**
     * Returns the path to the template settings file, based on the shop root directory.
     *
     * @return float
     */
    public function getThemeVersion()
    {
        if ($this->themeVersion === null) {
            $this->themeVersion = 1.0;
            if ($this->isThemeSystemActive() && array_key_exists('THEME_PRESENTATION_VERSION',
                    $this->getThemeSettings())) {
                $this->themeVersion = $this->getThemeSettings()['THEME_PRESENTATION_VERSION'];
            } elseif (array_key_exists('TEMPLATE_PRESENTATION_VERSION', $this->getThemeSettings())) {
                $this->themeVersion = $this->getThemeSettings()['TEMPLATE_PRESENTATION_VERSION'];
            }
        }

        return $this->themeVersion;
    }


    /**
     * Determines the current theme name and theme system status.
     */
    protected function _determineCurrentThemeStatus()
    {
        $this->themeSystemActive = false;
        $this->currentTheme = $this->currentTemplate;

        $themeConfig = $this->db->select('configuration_value AS value')
            ->from('configuration')
            ->where('configuration_key', 'CURRENT_THEME')
            ->get()
            ->row_array();
        if ($themeConfig !== null) {
            if ($themeConfig['value'] != '' && is_dir(DIR_FS_CATALOG . 'themes/' . $themeConfig['value'])) {
                $this->themeSystemActive = true;
                $this->currentTheme = $themeConfig['value'];
            }
        }
    }
}
